<?php

namespace Drupal\domain_microsite;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\PathProcessor\InboundPathProcessorInterface;
use Drupal\Core\PathProcessor\OutboundPathProcessorInterface;
use Drupal\Core\Render\BubbleableMetadata;
use Drupal\domain\DomainNegotiatorInterface;
use Drupal\path_alias\AliasManagerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Processes inbound and outbound paths for domain microsites.
 */
class DomainMicrositePathProcessor implements OutboundPathProcessorInterface, InboundPathProcessorInterface {

  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Domain negotiator.
   *
   * @var \Drupal\domain\DomainNegotiatorInterface
   */
  protected $domainNegotiator;

  /**
   * Alias manager.
   *
   * @var \Drupal\path_alias\AliasManagerInterface
   */
  protected $aliasManager;

  /**
   * Module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * DomainMicrositePathProcessor constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\domain\DomainNegotiatorInterface $domain_negotiator
   *   The domain negotiator.
   * @param \Drupal\path_alias\AliasManagerInterface $alias_manager
   *   The path alias manager.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, DomainNegotiatorInterface $domain_negotiator, AliasManagerInterface $alias_manager, ModuleHandlerInterface $module_handler) {
    $this->entityTypeManager = $entity_type_manager;
    $this->domainNegotiator = $domain_negotiator;
    $this->aliasManager = $alias_manager;
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   */
  public function processInbound($path, Request $request) {
    static $processed;
    $_path = $path;
    if (isset($processed[$_path])) {
      return $processed[$_path];
    }
    $microsite_path = domain_microsite_base_path();
    if ($microsite_path != '' && strpos($path, '/' . $microsite_path) === 0) {
      $new_path = substr($path, strlen('/' . $microsite_path));
      if ($new_path == '') {
        $path = '/';
      }
      // Only modify the path if the domain microsite base path matched whole
      // word path parts.
      elseif (strpos($new_path, '/') === 0) {
        $path = $new_path;
      }
    }
    $processed[$_path] = $path;
    return $path;
  }

  /**
   * {@inheritdoc}
   */
  public function processOutbound($path, &$options = [], Request $request = NULL, BubbleableMetadata $bubbleable_metadata = NULL) {
    static $processed;
    $_path = $path;
    if (isset($processed[$_path])) {
      return $processed[$_path];
    }
    if ($domain = $this->domainNegotiator->getActiveDomain()) {
      // If an entity is not assigned to the active domain or all affiliates,
      // the path needs to be altered using an assigned domain or else the path
      // will be inaccessible to non-admins.
      $entity = $options['entity'] ?? NULL;
      if ($entity instanceof ContentEntityInterface && $entity->hasField('field_domain_access')) {
        $assigned_domains = array_column($entity->get('field_domain_access')->getValue(), 'target_id');
        $all_affiliates = $entity->get('field_domain_all_affiliates')->value;
        if ($all_affiliates == FALSE && !empty($assigned_domains) && !in_array($domain->id(), $assigned_domains)) {
          $new_domain = $this->entityTypeManager->getStorage('domain')->load($assigned_domains[0]);
          // Using the base url option can interfere with other modules,
          // such as the Language module domain url negotiation.
          $options['base_url'] = trim($new_domain->getPath(), '/');
          // Get the domain path alias on the new domain.
          // Domain_path 8.x-1.0 switching from path processors to path
          // manager alias override is why this is like this.
          if ($this->moduleHandler->moduleExists('domain_path')) {
            $source = $this->aliasManager->getPathByAlias($path);
            $this->domainNegotiator->setActiveDomain($new_domain);
            $path = $this->aliasManager->getAliasByPath($source);
            $this->domainNegotiator->setActiveDomain($domain);
          }
        }
      }
      // Append microsite base path if node is assigned to active domain.
      if (empty($new_domain) && $base_path = $domain->getThirdPartySetting('domain_microsite', 'base_path')) {
        $path = $base_path . $path;
      }
    }
    if ($bubbleable_metadata) {
      $bubbleable_metadata->addCacheContexts(['domain_microsite']);
    }
    $processed[$_path] = $path;
    return $path;
  }

}
